@@ -4,25 +4,26 @@ use strict;
 require Exporter;
 our @ISA = qw(Exporter);
 our @SVG_REGEX = qw/
-		       $closepath
-		       $curveto
-		       $smooth_curveto
-		       $drawto_command
-		       $drawto_commands
-		       $elliptical_arc
-		       $horizontal_lineto
-		       $lineto
-		       $moveto
-		       $quadratic_bezier_curveto
-		       $smooth_quadratic_bezier_curveto
-		       $svg_path
-		       $vertical_lineto
-		   /;
+    $closepath
+    $curveto
+    $smooth_curveto
+    $drawto_command
+    $drawto_commands
+    $elliptical_arc
+    $horizontal_lineto
+    $lineto
+    $moveto
+    $quadratic_bezier_curveto
+    $smooth_quadratic_bezier_curveto
+    $svg_path
+    $vertical_lineto
+/;
 
 our @FUNCTIONS = qw/extract_path_info reverse_path create_path_string/;
 our @EXPORT_OK = (@FUNCTIONS, @SVG_REGEX);
 our %EXPORT_TAGS = (all => \@FUNCTIONS, regex => \@SVG_REGEX);
-our $VERSION = '0.34';
+
+our $VERSION = '0.35';
 
 use Carp;
 
@@ -175,22 +176,21 @@ my $exponent = qr/$e$sign?$ds/x;
 # Floating point constant
 
 my $fpc = qr/
-		$fc 
-		$exponent?
-	    |
-		$ds
-		$exponent
-	    /x;
-
+    $fc 
+    $exponent?
+|
+    $ds
+    $exponent
+/x;
 
 # Non-negative number. $floating_point_constant needs to go before
 # $ds, otherwise it matches the shorter $ds every time.
 
 my $nnn = qr/
-		$fpc
-	    |
-		$ds
-	    /x;
+    $fpc
+|
+    $ds
+/x;
 
 my $number = qr/$sign?$nnn/;
 
@@ -204,68 +204,83 @@ my $numbers = qr/(?:$number$wsp)*$number/;
 
 my $qarg = qr/$pair$comma_wsp?$pair/;
 
-our $quadratic_bezier_curveto = 
-qr/
-      ([Qq])
-      $wsp*
-      (
-	  (?:$qarg $comma_wsp?)*
-	  $qarg
-      )
-  /x;
+our $quadratic_bezier_curveto = qr/
+    ([Qq])
+    $wsp*
+    (
+	(?:$qarg $comma_wsp?)*
+	$qarg
+    )
+/x;
 
 our $smooth_quadratic_bezier_curveto =
 qr/
-      ([Tt])
-      $wsp*
-      (
-	  (?:$pair $comma_wsp?)*
-	  $pair
-      )
-  /x;
+    ([Tt])
+    $wsp*
+    (
+	(?:$pair $comma_wsp?)*
+	$pair
+    )
+/x;
 
 # Cubic bezier curve
 
 my $sarg = qr/$pair$comma_wsp?$pair/;
 
 our $smooth_curveto = qr/
-			    ([Ss])
-			    $wsp*
-			    (
-				(?:
-				    $sarg
-				    $comma_wsp
-				)*
-				$sarg
-			    )
-			/x;
+    ([Ss])
+    $wsp*
+    (
+	(?:
+	    $sarg
+	    $comma_wsp
+	)*
+	$sarg
+    )
+/x;
 
 my $carg = qr/(?:$pair $comma_wsp?){2} $pair/x;
 
 our $curveto = qr/
-		     ([Cc]) 
-		     $wsp*
-		     (
-			 (?:$carg $comma_wsp)*
-			 $carg
-		     )
-		 /x;
+    ([Cc]) 
+    $wsp*
+    (
+	(?:$carg $comma_wsp)*
+	$carg
+    )
+/x;
 
 my $flag = qr/[01]/;
 
+my $cpair = qr/($number)$comma_wsp?($number)/;
+
+my $ceaa = qr/
+    ($nnn)
+    $comma_wsp?
+    ($nnn)
+    $comma_wsp?
+    ($number)
+    $comma_wsp
+    ($flag)
+    $comma_wsp?
+    ($flag)
+    $comma_wsp?
+    $cpair
+/x;
+
 my $eaa = qr/
-		$nnn
-		$comma_wsp?
-		$nnn
-		$comma_wsp?
-		$number
-		$comma_wsp
-		$flag
-		$comma_wsp?
-		$flag
-		$comma_wsp?
-		$pair
-	    /x;
+    $nnn
+    $comma_wsp?
+    $nnn
+    $comma_wsp?
+    $number
+    $comma_wsp
+    $flag
+    $comma_wsp?
+    $flag
+    $comma_wsp?
+    $cpair
+/x;
 
 our $elliptical_arc = qr/([Aa]) $wsp* ((?:$eaa $comma_wsp?)* $eaa)/x;
 
@@ -278,52 +293,53 @@ our $lineto = qr/([Ll]) $wsp* ($pairs)/x;
 our $closepath = qr/([Zz])/;
 
 our $moveto = qr/
-		    ([Mm]) $wsp* ($pairs)
-		/x;
+    ([Mm]) $wsp* ($pairs)
+/x;
 
 our $drawto_command = qr/
-			    (
-				$closepath
-			    |
-				$lineto
-			    |
-				$horizontal_lineto
-			    |
-				$vertical_lineto
-			    |
-				$curveto
-			    |
-				$smooth_curveto
-			    |
-				$quadratic_bezier_curveto
-			    |
-				$smooth_quadratic_bezier_curveto
-			    |
-				$elliptical_arc
-			    )
-			/x;
+    (
+	$closepath
+    |
+	$lineto
+    |
+	$horizontal_lineto
+    |
+	$vertical_lineto
+    |
+	$curveto
+    |
+	$smooth_curveto
+    |
+	$quadratic_bezier_curveto
+    |
+	$smooth_quadratic_bezier_curveto
+    |
+	$elliptical_arc
+    )
+/x;
 
 our $drawto_commands = qr/
-			     (?:$drawto_command $wsp)*
-			     $drawto_command
-			 /x;
+    (?:$drawto_command $wsp)*
+    $drawto_command
+/x;
+
 our $mdc_group = qr/
-		      $moveto
-		      $wsp*
-		      $drawto_commands
-		  /x;
+    $moveto
+    $wsp*
+    $drawto_commands
+/x;
 
 my $mdc_groups = qr/
-		       $mdc_group+
-		   /x;
+    $mdc_group+
+/x;
 
 our $moveto_drawto_command_groups = $mdc_groups;
 
 our $svg_path = qr/
-		      $wsp*
-		      $mdc_groups?
-		      $wsp*
-		  /x;
+    $wsp*
+    $mdc_groups?
+    $wsp*
+/x;
 
 # Old regex.
 
@@ -367,7 +383,7 @@ sub extract_path_info
     shift @path;
     my $path_pos=0;
     my @curves;
-    while ( $path_pos < scalar @path ) {
+    while ($path_pos < scalar @path) {
         my $command = $path[$path_pos];
         my $values = $path[$path_pos+1];
 	if (! defined $values) {
@@ -383,8 +399,15 @@ sub extract_path_info
     }
     for my $curve_data (@curves) {
         my ($command, $values) = @$curve_data;
+	my $ucc = uc $command;
 #	print "$curve\n";
-        my @numbers = ($values =~ /($number)/g);
+        my @numbers;
+	if ($ucc eq 'A') {
+	    @numbers = ($values =~ /$ceaa/g);
+	}
+	else {
+	    @numbers = ($values =~ /($number)/g);
+	}
 	# Remove leading plus signs to keep the same behaviour as
 	# before.
 	@numbers = map {s/^\+//; $_} @numbers;
@@ -393,7 +416,7 @@ sub extract_path_info
             printf "$me: Extracted %d numbers: %s\n", scalar (@numbers),
 	    join (" ! ", @numbers);
         }
-        if (uc $command eq 'C') {
+        if ($ucc eq 'C') {
             my $expect_numbers = 6;
             if (@numbers % $expect_numbers != 0) {
                 croak "$me: Wrong number of values for a C curve " .
@@ -528,7 +551,8 @@ sub extract_path_info
             my $position = position_type ($command);
             my $expect_numbers = 7;
 	    if (@numbers % $expect_numbers != 0) {
-		croak "$me: Need 7 parameters for arc";
+		my $n = scalar (@numbers);
+		croak "$me: Need multiple of 7 parameters for arc, got $n (@numbers)";
 	    }
             for (my $i = 0; $i < @numbers / $expect_numbers; $i++) {
                 my $o = $expect_numbers * $i;
